"""
CSVTool Product for Zope

TODO: Description

"""
import csv, os
from cStringIO import StringIO
from Globals import package_home, InitializeClass, DTMLFile
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo

modulesecurity = ModuleSecurityInfo()
DEFAULT_PERM = 'Use CSV Tool'

class CSVTool(SimpleItem):
	""" CSV Tool """
	meta_type	= 'CSV Tool'
	title		= ''

	_properties =	(
		{'id':'title','type':'string','mode':'w'},
		{'id':'mode','type':'string','mode':'w'},
		{'id':'delimiter','type':'string','mode':'w'},
			)
	property_extensible_schema__ = 0

	manage_options = (
			{'label': 'Settings', 'action':'manage_csvtool'},
		) + SimpleItem.manage_options

	mode = 'reader (single line)'
	mode_list = (
		'reader (single line)',
		'reader (multi line)',
		'writer (single line)',
		'writer (multi line)',
	)
	quoting = 'QUOTE_MINIMAL'
	quoting_list = (
		'QUOTE_MINIMAL',
		'QUOTE_ALL',
		'QUOTE_NONE',
		'QUOTE_NONNUMERIC',
	)
	delimiter = 'camma'
	delimiter_list = (
		'camma',
		'space',
		'tab',
		'colon',
		'semicolon',
	)
	quotechar = 'doublequote'
	quotechar_list = (
		'doublequote',
		'singlequote',
	)

	security	= ClassSecurityInfo()

	security.declareProtected(DEFAULT_PERM, 'manage_csvtool')
	manage_csvtool = DTMLFile('dtml/manageCSVTool', globals())

	#------------
	# interface
	#------------
	security.declareProtected(DEFAULT_PERM, '__call__')
	def __call__(self, arg):
		""" """
		mode = self.mode
		if mode == 'reader (single line)':
			return self.read_single_line(arg)
		elif mode == 'reader (multi line)':
			return self.read_multi_line(arg)
		elif mode == 'writer (single line)':
			return self.write_single_line(arg)
		elif mode == 'writer (multi line)':
			return self.write_multi_line(arg)

	security.declareProtected(DEFAULT_PERM, 'manage_changeCSVTool')
	def manage_changeCSVTool(self, title=None, mode=None, quoting=None, delimiter=None, quotechar=None, REQUEST=None):
		""" change CSVTool settings """
		if title:
			self.title = title
		if mode:
			self.setMode(mode)
		if quoting:
			self.setQuoting(quoting)
		if delimiter:
			self.setDelimiter(delimiter)
		if quotechar:
			self.setQuotechar(quotechar)
		if REQUEST is not None:
			return self.manage_csvtool(
				self, REQUEST, manage_tabs_message='Changes saved.'
			)

	#---------------------------
	# accessor
	#---------------------------

	security.declareProtected(DEFAULT_PERM, 'getMode')
	def getMode(self):
		return self.mode

	security.declareProtected(DEFAULT_PERM, 'setMode')
	def setMode(self, mode):
		if not mode in self.mode_list:
			raise 'Invalid mode %s' %(mode)
		self.mode = mode

	security.declareProtected(DEFAULT_PERM, 'getQuoting')
	def getQuoting(self):
		return self.quoting

	security.declareProtected(DEFAULT_PERM, 'setQuoting')
	def setQuoting(self, quoting):
		if not quoting in self.quoting_list:
			raise 'Invalid quoting %s' %(quoting)
		self.quoting = quoting

	security.declareProtected(DEFAULT_PERM, 'getDelimiter')
	def getDelimiter(self):
		return self.delimiter

	security.declareProtected(DEFAULT_PERM, 'setDelimiter')
	def setDelimiter(self, delimiter):
		if not delimiter in self.delimiter_list:
			raise 'Invalid delimiter %s' %(delimiter)
		self.delimiter = delimiter

	security.declareProtected(DEFAULT_PERM, 'getQuotechar')
	def getQuotechar(self):
		return self.quotechar

	security.declareProtected(DEFAULT_PERM, 'setQuotechar')
	def setQuotechar(self, quotechar):
		if not quotechar in self.quotechar_list:
			raise 'Invalid quotechar %s' %(quotechar)
		self.quotechar = quotechar

	#---------------------------
	# real job method (private)
	#---------------------------
	security.declarePrivate('read_single_line')
	def read_single_line(self, txt):
		""" act like csv reader """
		sio = StringIO()
		sio.write(txt)
		sio.seek(0)
		q, d, c = self._getOptions()
		reader = csv.reader(sio, quoting=q, delimiter=d, quotechar=c)
		for l in reader:
			return l
		return None

	security.declarePrivate('read_multi_line')
	def read_multi_line(self, txt):
		""" act like csv reader """
		sio = StringIO()
		sio.write(txt)
		sio.seek(0)
		q, d, c = self._getOptions()
		reader = csv.reader(sio, quoting=q, delimiter=d, quotechar=c)
		list_of_list = []
		for l in reader:
			list_of_list.append(l)
		return list_of_list

	security.declarePrivate('write_single_line')
	def write_single_line(self, lst):
		""" act like csv writer """
		sio = StringIO()
		q, d, c = self._getOptions()
		writer = csv.writer(sio, quoting=q, delimiter=d, quotechar=c)
		writer.writerow(lst)
		return sio.getvalue().strip()

	security.declarePrivate('write_multi_line')
	def write_multi_line(self, lstoflst):
		""" act like csv writer """
		sio = StringIO()
		q, d, c = self._getOptions()
		writer = csv.writer(sio, quoting=q, delimiter=d, quotechar=c)
		for lst in lstoflst:
			writer.writerow(lst)
		return sio.getvalue().strip()

	#---------------------------
	# misc pribvate method
	#---------------------------

	def _getOptions(self):
		q	=  self._getRealQuoting()
		d	=  self._getRealDelimiter()
		c	=  self._getRealQuotechar()
		return (q, d, c)

	def _getRealQuoting(self):
		d = {
			'QUOTE_MINIMAL'		: csv.QUOTE_MINIMAL,
			'QUOTE_ALL'			: csv.QUOTE_ALL,
			'QUOTE_NONE'		: csv.QUOTE_NONE,
			'QUOTE_NONNUMERIC'	: csv.QUOTE_NONNUMERIC,
			}
		return d[self.quoting]

	def _getRealDelimiter(self):
		d = {
			'camma'		: ',',
			'space'		: ' ',
			'tab'		: '	',
			'colon'		: ':',
			'semicolon'	: ';',
			}
		return d[self.delimiter]

	def _getRealQuotechar(self):
		d = {
			'singlequote'	: '\'',
			'doublequote'	: '\"',
			}
		return d[self.quotechar]

InitializeClass(CSVTool)

modulesecurity.declareProtected(
	'View management screens','manage_addCSVToolForm')
manage_addCSVToolForm = DTMLFile('dtml/manage_addCSVToolForm', globals())

def manage_addCSVTool(self, new_id, title='', create_sample=None, REQUEST=None):
	"""add CSV Too """
	obj = CSVTool()
	obj.id = str(new_id)
	obj.title = str(title)
	id = self._setObject(obj.id, obj)
	obj = getattr(self, id)
	if create_sample:
		py_id = obj.id + '_usage_sample'
		self.manage_addProduct['PythonScripts'].manage_addPythonScript(py_id)
		py_obj = getattr(self, py_id)
		sample_path = os.path.join(package_home(globals()),'files','usage_sample.py')
		sample_src = open(sample_path).read()
		sample_src = sample_src.replace('CSVTOOL_OBJECT_ID', obj.id)
		py_obj.write(sample_src)
	if REQUEST:
		return self.manage_main(self, REQUEST)

modulesecurity.apply(globals())
